﻿using Abt.Controls.SciChart;
using Abt.Controls.SciChart.Model.DataSeries;
using Abt.Controls.SciChart.Visuals.RenderableSeries;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Timers;
using System.Windows.Input;
using System.Windows.Media;

namespace ScimultiPane
{
	/// <summary>
	/// Область графика
	/// </summary>
	public abstract class BaseChartPaneViewModel : BaseViewModel, IChildPane
	{
		private readonly CreateMultiPaneStockChartsViewModel _parentViewModel;
		private readonly ObservableCollection<IChartSeriesViewModel> _chartSeriesViewModels = new ObservableCollection<IChartSeriesViewModel>();
		private string _title;
		private string _yAxisTextFormatting;
		private bool _isFirstChartPane;
		private bool _isLastChartPane;
		private double _height = double.NaN;

		protected BaseChartPaneViewModel(CreateMultiPaneStockChartsViewModel parentViewModel)
		{
			_parentViewModel = parentViewModel;

			ViewportManager = new DefaultViewportManager();
		}

		public CreateMultiPaneStockChartsViewModel ParentViewModel
		{
			get { return _parentViewModel; }
		}

		public ObservableCollection<IChartSeriesViewModel> ChartSeriesViewModels
		{
			get { return _chartSeriesViewModels; }
		}

		public IViewportManager ViewportManager { get; set; }

		public string YAxisTextFormatting
		{
			get { return _yAxisTextFormatting; }
			set
			{
				if (_yAxisTextFormatting == value)
					return;

				_yAxisTextFormatting = value;
				OnPropertyChanged("YAxisTextFormatting");
			}
		}

		public string Title
		{
			get { return _title; }
			set
			{
				if (_title == value) return;

				_title = value;
				OnPropertyChanged("Title");
			}
		}

		public bool IsFirstChartPane
		{
			get { return _isFirstChartPane; }
			set
			{
				if (_isFirstChartPane == value) return;
				_isFirstChartPane = value;
				OnPropertyChanged("IsFirstChartPane");
			}
		}

		public bool IsLastChartPane
		{
			get { return _isLastChartPane; }
			set
			{
				if (_isLastChartPane == value) return;
				_isLastChartPane = value;
				OnPropertyChanged("IsLastChartPane");
			}
		}

		public double Height
		{
			get { return _height; }
			set
			{
				if (Math.Abs(_height - value) < double.Epsilon) return;
				_height = value;
				OnPropertyChanged("Height");
			}
		}

		public void ZoomExtents()
		{
		}

		public ICommand ClosePaneCommand
		{
			get;
			set;
		}
	}
	public class CreateMultiPaneStockChartsViewModel : BaseViewModel
	{
		private IndexRange _xAxisVisibleRange;
		private ObservableCollection<BaseChartPaneViewModel> _chartPaneViewModels = new ObservableCollection<BaseChartPaneViewModel>();
		private readonly ICommand _closePaneCommand;
		private bool _isPanEnabled;
		private bool _isZoomEnabled;
		private string _verticalChartGroupId;
		private IViewportManager _viewportManager;

		public CreateMultiPaneStockChartsViewModel()
		{
			ZoomModeCommand = new ActionCommand(SetZoomMode);
			PanModeCommand = new ActionCommand(SetPanMode);
			ZoomExtentsCommand = new ActionCommand(ZoomExtends);

			// Get prices and append for the main price chart (Candlestick)
			var _marketDataService = new MarketDataService(new DateTime(2000, 08, 01, 12, 00, 00), 5, 20);
			var PriceData = _marketDataService.GetHistoricalData(100);
			PriceSeries instrumentPriceData = new PriceSeries();
			instrumentPriceData.AddRange(PriceData);

			// ChartGroup is an ID which is used to synchronize chart pane heights and mouse events. it must be unique per SciChartGroup, but differ if you have many top level SciChartGroups
			_verticalChartGroupId = Guid.NewGuid().ToString();

			_viewportManager = new DefaultViewportManager();
			var closePaneCommand = new ActionCommand<IChildPane>(pane => ChartPaneViewModels.Remove((BaseChartPaneViewModel)pane));

			_chartPaneViewModels.Add(new PricePaneViewModel(this, instrumentPriceData) { IsFirstChartPane = true, ViewportManager = _viewportManager });
			_chartPaneViewModels.Add(new MacdPaneViewModel(this, instrumentPriceData) { Title = "MACD", ClosePaneCommand = closePaneCommand });
			_chartPaneViewModels.Add(new RsiPaneViewModel(this, instrumentPriceData) { Title = "RSI", ClosePaneCommand = closePaneCommand });
			_chartPaneViewModels.Add(new VolumePaneViewModel(this, instrumentPriceData) { Title = "Volume", ClosePaneCommand = closePaneCommand, IsLastChartPane = true });

			SetZoomMode();
		}

		private void ZoomExtends()
		{
			_viewportManager.AnimateZoomExtents(TimeSpan.FromMilliseconds(500));
		}

		public IEnumerable<string> AllThemes { get { return ThemeManager.AllThemes; } }

		public ICommand ZoomModeCommand { get; private set; }

		public ICommand PanModeCommand { get; private set; }

		public ICommand ZoomExtentsCommand { get; private set; }

		public string VerticalChartGroupId
		{
			get { return _verticalChartGroupId; }
			set
			{
				if (_verticalChartGroupId == value) return;
				_verticalChartGroupId = value;
				OnPropertyChanged("VerticalChartGroupId");
			}
		}

		/// <summary>
		/// Shared XAxis VisibleRange for all charts
		/// </summary>
		public IndexRange XVisibleRange
		{
			get { return _xAxisVisibleRange; }
			set
			{
				if (Equals(_xAxisVisibleRange, value)) return;

				_xAxisVisibleRange = value;
				OnPropertyChanged("XVisibleRange");
			}
		}

		public ObservableCollection<BaseChartPaneViewModel> ChartPaneViewModels
		{
			get { return _chartPaneViewModels; }
			set
			{
				if (_chartPaneViewModels == value) return;

				_chartPaneViewModels = value;
				OnPropertyChanged("ChartPaneViewModels");
			}
		}

		public bool IsPanEnabled
		{
			get { return _isPanEnabled; }
			set
			{
				_isPanEnabled = value;
				OnPropertyChanged("IsPanEnabled");
			}
		}

		public bool IsZoomEnabled
		{
			get { return _isZoomEnabled; }
			set
			{
				_isZoomEnabled = value;
				OnPropertyChanged("IsZoomEnabled");
			}
		}

		private void SetPanMode()
		{
			IsPanEnabled = true;
			IsZoomEnabled = false;
		}

		private void SetZoomMode()
		{
			IsPanEnabled = false;
			IsZoomEnabled = true;
		}
	}

	public class RandomPricesDataSource
	{
		private sealed class PriceBarInfo
		{
			public DateTime DateTime;
			public double Close;
		}

		private const double Frequency = 1.1574074074074073E-05;
		private readonly Random _random;
		private readonly int _candleIntervalMinutes;
		private readonly bool _simulateDateGap;
		private readonly Timer _timer;
		private PriceBar _lastPriceBar;
		private readonly PriceBarInfo _initialPriceBar;
		private double _currentTime;
		private readonly int _updatesPerPrice;
		private int _currentUpdateCount;
		private readonly TimeSpan _openMarketTime = new TimeSpan(0, 08, 0, 0);
		private readonly TimeSpan _closeMarketTime = new TimeSpan(0, 16, 30, 0);
		public event OnNewData NewData;
		public event OnUpdateData UpdateData;

		public RandomPricesDataSource(int candleIntervalMinutes, bool simulateDateGap, double timerInterval, int updatesPerPrice, int randomSeed, double startingPrice, DateTime startDate)
		{
			_candleIntervalMinutes = candleIntervalMinutes;
			_simulateDateGap = simulateDateGap;
			_updatesPerPrice = updatesPerPrice;

			_timer = new Timer(timerInterval)
			{
				Enabled = false,
				AutoReset = true,
			};
			_timer.Elapsed += TimerElapsed;
			_initialPriceBar = new PriceBarInfo
			{
				Close = startingPrice,
				DateTime = startDate
			};
			_lastPriceBar = new PriceBar(_initialPriceBar.DateTime, _initialPriceBar.Close, _initialPriceBar.Close, _initialPriceBar.Close, _initialPriceBar.Close, 0L);
			_random = new Random(randomSeed);
		}

		public bool IsRunning
		{
			get { return _timer.Enabled; }
		}

		public void StartGeneratePriceBars()
		{
			_timer.Enabled = true;
		}

		public void StopGeneratePriceBars()
		{
			_timer.Enabled = false;
		}

		public PriceBar GetNextData()
		{
			PriceBar nextRandomPriceBar = GetNextRandomPriceBar();
			return nextRandomPriceBar;
		}

		private PriceBar GetNextRandomPriceBar()
		{
			double close = _lastPriceBar.Close;
			double num = (_random.NextDouble() - 0.9) * _initialPriceBar.Close / 30.0;
			double num2 = _random.NextDouble();
			double num3 = _initialPriceBar.Close + _initialPriceBar.Close / 2.0 * Math.Sin(7.27220521664304E-06 * _currentTime) + _initialPriceBar.Close / 16.0 * Math.Cos(7.27220521664304E-05 * _currentTime) + _initialPriceBar.Close / 32.0 * Math.Sin(7.27220521664304E-05 * (10.0 + num2) * _currentTime) + _initialPriceBar.Close / 64.0 * Math.Cos(7.27220521664304E-05 * (20.0 + num2) * _currentTime) + num;
			double num4 = Math.Max(close, num3);
			double num5 = _random.NextDouble() * _initialPriceBar.Close / 100.0;
			double high = num4 + num5;
			double num6 = Math.Min(close, num3);
			double num7 = _random.NextDouble() * _initialPriceBar.Close / 100.0;
			double low = num6 - num7;
			long volume = (long)(_random.NextDouble() * 30000 + 20000);
			DateTime openTime = _simulateDateGap ? EmulateDateGap(_lastPriceBar.DateTime) : _lastPriceBar.DateTime;
			DateTime closeTime = openTime.AddMinutes(_candleIntervalMinutes);
			PriceBar candle = new PriceBar(closeTime, close, high, low, num3, volume);
			_lastPriceBar = new PriceBar(candle.DateTime, candle.Open, candle.High, candle.Low, candle.Close, volume);
			_currentTime += _candleIntervalMinutes * 60;
			return candle;
		}

		private DateTime EmulateDateGap(DateTime candleOpenTime)
		{
			DateTime result = candleOpenTime;
			TimeSpan timeOfDay = candleOpenTime.TimeOfDay;
			if (timeOfDay > _closeMarketTime)
			{
				DateTime dateTime = candleOpenTime.Date;
				dateTime = dateTime.AddDays(1.0);
				result = dateTime.Add(_openMarketTime);
			}
			while (result.DayOfWeek == DayOfWeek.Saturday || result.DayOfWeek == DayOfWeek.Sunday)
			{
				result = result.AddDays(1.0);
			}
			return result;
		}

#if SILVERLIGHT

        private void TimerElapsed(object sender, EventArgs e)
        {
            OnTimerElapsed();
        }

#else

		private void TimerElapsed(object sender, ElapsedEventArgs e)
		{
			OnTimerElapsed();
		}

#endif

		private void OnTimerElapsed()
		{
			if (_currentUpdateCount < _updatesPerPrice)
			{
				_currentUpdateCount++;
				PriceBar updatedData = GetUpdatedData();
				if (UpdateData != null)
				{
					UpdateData(updatedData);
				}
			}
			else
			{
				_currentUpdateCount = 0;
				PriceBar nextData = GetNextData();
				if (NewData != null)
				{
					NewData(nextData);
				}
			}
		}

		private PriceBar GetUpdatedData()
		{
			double num = _lastPriceBar.Close + ((_random.NextDouble() - 0.48) * (_lastPriceBar.Close / 100.0));
			double high = (num > _lastPriceBar.High) ? num : _lastPriceBar.High;
			double low = (num < _lastPriceBar.Low) ? num : _lastPriceBar.Low;
			long volumeInc = (long)((_random.NextDouble() * 30000 + 20000) * 0.05);
			_lastPriceBar = new PriceBar(_lastPriceBar.DateTime, _lastPriceBar.Open, high, low, num, _lastPriceBar.Volume + volumeInc);

			return _lastPriceBar;
		}

		public void ClearEventHandlers()
		{
			NewData = null;
			UpdateData = null;
		}

		public PriceBar Tick()
		{
			if (_currentUpdateCount < _updatesPerPrice)
			{
				_currentUpdateCount++;
				return GetUpdatedData();
			}
			else
			{
				_currentUpdateCount = 0;
				return GetNextData();
			}
		}
	}
	public interface IMarketDataService
	{
		void SubscribePriceUpdate(Action<PriceBar> callback);
		IEnumerable<PriceBar> GetHistoricalData(int numberBars);
		void ClearSubscriptions();
		PriceBar GetNextBar();
	}

	public delegate void OnNewData(PriceBar data);
	public delegate void OnUpdateData(PriceBar data);

	public class MarketDataService : IMarketDataService
	{
		private readonly DateTime _startDate;
		private readonly int _timeFrameMinutes;
		private readonly int _tickTimerIntervalms;
		private readonly RandomPricesDataSource _generator;

		public MarketDataService(DateTime startDate, int timeFrameMinutes, int tickTimerIntervalms)
		{
			_startDate = startDate;
			_timeFrameMinutes = timeFrameMinutes;
			_tickTimerIntervalms = tickTimerIntervalms;
			_generator = new RandomPricesDataSource(_timeFrameMinutes, true, _tickTimerIntervalms, 25, 367367, 30, _startDate);
		}

		public void SubscribePriceUpdate(Action<PriceBar> callback)
		{
			if (!_generator.IsRunning)
			{
				_generator.NewData += (arg) => callback(arg);
				_generator.UpdateData += (arg) => callback(arg);

				_generator.StartGeneratePriceBars();
			}
		}

		public IEnumerable<PriceBar> GetHistoricalData(int numberBars)
		{
			List<PriceBar> prices = new List<PriceBar>(numberBars);
			for (int i = 0; i < numberBars; i++)
			{
				prices.Add(_generator.GetNextData());
			}

			return prices;
		}

		public void ClearSubscriptions()
		{
			if (_generator.IsRunning)
			{
				_generator.StopGeneratePriceBars();
				_generator.ClearEventHandlers();
			}
		}

		public PriceBar GetNextBar()
		{
			return _generator.Tick();
		}
	}
	public class XYPoint
	{
		public double X { get; set; }
		public double Y { get; set; }
	}
	public class DoubleSeries : List<XYPoint>
	{
		public DoubleSeries()
		{
		}

		public DoubleSeries(int capacity)
			: base(capacity)
		{
		}

		public IList<double> XData { get { return this.Select(x => x.X).ToArray(); } }
		public IList<double> YData { get { return this.Select(x => x.Y).ToArray(); } }
	}
	public interface IDataManager
	{
		PriceSeries GetPriceData(string symbol, TimeFrame timeFrame);
		DoubleSeries GetFourierSeries(double amplitude, double phaseShift, int count = 5000);
		DoubleSeries GetSquirlyWave();

		IEnumerable<Instrument> AvailableInstruments { get; }
		IEnumerable<TimeFrame> GetAvailableTimeFrames(Instrument forInstrument);
	}
	public class Instrument : StrongTyped<string>
	{
		public string InstrumentName { get; private set; }
		public string Symbol { get { return Value; } }
		public int DecimalPlaces { get; private set; }

		public Instrument(string value, string instrumentName, int decimalPlaces)
			: base(value)
		{
			InstrumentName = instrumentName;
			DecimalPlaces = decimalPlaces;
		}

		public static Instrument Parse(string instrumentString)
		{
			return new[] { EurUsd, Indu, Spx500, CrudeOil, Test }.Single(x => x.Symbol.ToUpper(CultureInfo.InvariantCulture) == instrumentString.ToUpper(CultureInfo.InvariantCulture));
		}

		public static readonly Instrument EurUsd = new Instrument("EURUSD", "FX Euro US Dollar", 4);
		public static readonly Instrument Indu = new Instrument("INDU", "Dow Jones Industrial Average", 0);
		public static readonly Instrument Spx500 = new Instrument("SPX500", "S&P500 Index", 0);
		public static readonly Instrument CrudeOil = new Instrument("CL", "Light Crude Oil", 0);
		public static readonly Instrument Test = new Instrument("TEST", "Test data only", 0);
	}
	public class TimeFrame : StrongTyped<string>
	{
		public TimeFrame(string value, string displayname)
			: base(value)
		{
			Displayname = displayname;
		}

		public static readonly TimeFrame Daily = new TimeFrame("Daily", "Daily");
		public static readonly TimeFrame Hourly = new TimeFrame("Hourly", "1 Hour");
		public static readonly TimeFrame Minute15 = new TimeFrame("Minute15", "15 Minutes");

		public string Displayname { get; private set; }

		public static TimeFrame Parse(string input)
		{
			return new[] { Minute15, Hourly, Daily }.Single(x => x.Value.ToUpper(CultureInfo.InvariantCulture) == input.ToUpper(CultureInfo.InvariantCulture));
		}
	}
	public class StrongTyped<T> : IEquatable<StrongTyped<T>>
	{
		public T Value { get; protected set; }

		public StrongTyped(T value)
		{
			Value = value;
		}

		public override bool Equals(object obj)
		{
			return base.Equals(obj);
		}

		public bool Equals(StrongTyped<T> other)
		{
			if (ReferenceEquals(null, other)) return false;
			if (ReferenceEquals(this, other)) return true;
			return Equals(other.Value, Value);
		}

		public override int GetHashCode()
		{
			return Value.GetHashCode();
		}

		public static bool operator ==(StrongTyped<T> left, StrongTyped<T> right)
		{
			return Equals(left, right);
		}

		public static bool operator !=(StrongTyped<T> left, StrongTyped<T> right)
		{
			return !Equals(left, right);
		}

		public override string ToString()
		{
			return Value.ToString();
		}
	}
	public class VolumePaneViewModel : BaseChartPaneViewModel
	{
		public VolumePaneViewModel(CreateMultiPaneStockChartsViewModel parentViewModel, PriceSeries prices)
			: base(parentViewModel)
		{
			var volumePrices = new XyDataSeries<DateTime, double>();
			volumePrices.Append(prices.TimeData, prices.VolumeData.Select(x => (double)x));
			base.ChartSeriesViewModels.Add(new ChartSeriesViewModel(volumePrices, new FastColumnRenderableSeries()));

			YAxisTextFormatting = "###E+0";
		}
	}
	public class MacdPaneViewModel : BaseChartPaneViewModel
	{
		public MacdPaneViewModel(CreateMultiPaneStockChartsViewModel parentViewModel, PriceSeries prices)
			: base(parentViewModel)
		{
			IEnumerable<MacdPoint> macdPoints = prices.CloseData.Macd(12, 26, 9).ToList();

			var histogramDataSeries = new XyDataSeries<DateTime, double>();
			histogramDataSeries.Append(prices.TimeData, macdPoints.Select(x => x.Divergence));
			base.ChartSeriesViewModels.Add(new ChartSeriesViewModel(histogramDataSeries, new FastColumnRenderableSeries()));

			var macdDataSeries = new XyyDataSeries<DateTime, double>();
			macdDataSeries.Append(prices.TimeData, macdPoints.Select(x => x.Macd), macdPoints.Select(x => x.Signal));
			base.ChartSeriesViewModels.Add(new ChartSeriesViewModel(macdDataSeries, new FastBandRenderableSeries() { Opacity = 0.7, StrokeThickness = 2 }));

			YAxisTextFormatting = "0.00";

			Height = 100;
		}
	}

	public class RsiPaneViewModel : BaseChartPaneViewModel
	{
		public RsiPaneViewModel(CreateMultiPaneStockChartsViewModel parentViewModel, PriceSeries prices)
			: base(parentViewModel)
		{
			var rsiSeries = new XyDataSeries<DateTime, double>();
			rsiSeries.Append(prices.TimeData, prices.Rsi(14));
			base.ChartSeriesViewModels.Add(new ChartSeriesViewModel(rsiSeries, new FastLineRenderableSeries()));

			YAxisTextFormatting = "0.0";

			Height = 100;
		}
	}

	public class PricePaneViewModel : BaseChartPaneViewModel
	{
		public PricePaneViewModel(CreateMultiPaneStockChartsViewModel parentViewModel, PriceSeries prices)
			: base(parentViewModel)
		{
			// We can add Series via the SeriesSource API, where SciStockChart or SciChartSurface bind to IEnumerable<IChartSeriesViewModel>
			// Alternatively, you can delcare your RenderableSeries in the SciStockChart and just bind to DataSeries
			// A third method (which we don't have an example for yet, but you can try out) is to create an Attached Behaviour to transform a collection of IDataSeries into IRenderableSeries
			// 

			// Add the main OHLC chart
			var stockPrices = new OhlcDataSeries<DateTime, double>();
			stockPrices.Append(prices.TimeData, prices.OpenData, prices.HighData, prices.LowData, prices.CloseData);
			base.ChartSeriesViewModels.Add(new ChartSeriesViewModel(stockPrices, new FastCandlestickRenderableSeries() { AntiAliasing = false }));

			// Add a moving average
			var maLow = new XyDataSeries<DateTime, double>();
			maLow.Append(prices.TimeData, prices.CloseData.MovingAverage(50));
			base.ChartSeriesViewModels.Add(new ChartSeriesViewModel(maLow, new FastLineRenderableSeries() { SeriesColor = Color.FromArgb(0xFF, 0xFF, 0x33, 0x33), StrokeThickness = 2 }));

			// Add a moving average
			var maHigh = new XyDataSeries<DateTime, double>();
			maHigh.Append(prices.TimeData, prices.CloseData.MovingAverage(200));
			base.ChartSeriesViewModels.Add(new ChartSeriesViewModel(maHigh, new FastLineRenderableSeries() { SeriesColor = Color.FromArgb(0xFF, 0x33, 0xDD, 0x33), StrokeThickness = 2 }));

			YAxisTextFormatting = "$0.0000";
		}
	}
	public class PriceBar
	{
		public PriceBar()
		{
		}

		public PriceBar(DateTime date, double open, double high, double low, double close, long volume)
		{
			DateTime = date;
			Open = open;
			High = high;
			Low = low;
			Close = close;
			Volume = volume;
		}

		public DateTime DateTime { get; set; }
		public double Open { get; set; }
		public double High { get; set; }
		public double Low { get; set; }
		public double Close { get; set; }
		public long Volume { get; set; }
	}
	public class PriceSeries : List<PriceBar>
	{
		public string Symbol { get; set; }

		public PriceSeries()
		{
		}

		public PriceSeries(int capacity)
			: base(capacity)
		{
		}

		/// <summary>
		/// Extracts the DateTime column of the PriceSeries as an array
		/// </summary>
		public IList<DateTime> TimeData { get { return this.Select(x => x.DateTime).ToArray(); } }

		/// <summary>
		/// Extracts the Open column of the PriceSeries as an array
		/// </summary>
		public IList<double> OpenData { get { return this.Select(x => x.Open).ToArray(); } }

		/// <summary>
		/// Extracts the High column of the PriceSeries as an array
		/// </summary>
		public IList<double> HighData { get { return this.Select(x => x.High).ToArray(); } }

		/// <summary>
		/// Extracts the Low column of the PriceSeries as an array
		/// </summary>
		public IList<double> LowData { get { return this.Select(x => x.Low).ToArray(); } }

		/// <summary>
		/// Extracts the Close column of the PriceSeries as an array
		/// </summary>
		public IList<double> CloseData { get { return this.Select(x => x.Close).ToArray(); } }

		/// <summary>
		/// Extracts the Volume column of the PriceSeries as an array
		/// </summary>
		public IList<long> VolumeData { get { return this.Select(x => x.Volume).ToArray(); } }

		public PriceSeries Clip(int startIndex, int endIndex)
		{
			var result = new PriceSeries(endIndex - startIndex);
			for (int i = startIndex; i < endIndex; i++)
			{
				result.Add(this[i]);
			}
			return result;
		}
	}
	public static class MovingAverageExtensions
	{
		/// <summary>
		/// Fast MovingAverage LINQ Extension method provided for example purposes on an AS IS BASIS ONLY. ACCURACY OF CALCULATION NOT GAURANTEED
		/// </summary>
		public static IEnumerable<double> MovingAverage<T>(this IEnumerable<T> inputStream, Func<T, double> selector, int period)
		{
			var ma = new MovingAverage(period);
			foreach (var item in inputStream)
			{
				ma.Push(selector(item));
				yield return ma.Current;
			}
		}

		/// <summary>
		/// Fast MovingAverage LINQ Extension method provided for example purposes on an AS IS BASIS ONLY. ACCURACY OF CALCULATION NOT GAURANTEED
		/// </summary>
		public static IEnumerable<double> MovingAverage(this IEnumerable<double> inputStream, int period)
		{
			var ma = new MovingAverage(period);
			foreach (var item in inputStream)
			{
				ma.Push(item);
				yield return ma.Current;
			}
		}

		/// <summary>
		/// Fast Macd LINQ Extension method provided for example purposes on an AS IS BASIS ONLY. ACCURACY OF CALCULATION NOT GAURANTEED
		/// </summary>
		public static IEnumerable<MacdPoint> Macd(this IEnumerable<double> inputStream, int slow, int fast, int signal)
		{
			var maSlow = new MovingAverage(slow);
			var maFast = new MovingAverage(fast);
			var maSignal = new MovingAverage(signal);

			foreach (var item in inputStream)
			{
				double macd = maSlow.Push(item).Current - maFast.Push(item).Current;
				double signalLine = double.IsNaN(macd) ? double.NaN : maSignal.Push(macd).Current;
				double divergence = double.IsNaN(macd) || double.IsNaN(signalLine) ? double.NaN : macd - signalLine;

				yield return new MacdPoint() { Macd = macd, Signal = signalLine, Divergence = divergence };
			}
		}

		/// <summary>
		/// Fast Rsi LINQ Extension method provided for example purposes on an AS IS BASIS ONLY. ACCURACY OF CALCULATION NOT GAURANTEED
		/// </summary>
		public static IEnumerable<double> Rsi(this IEnumerable<PriceBar> inputStream, int period)
		{
			var averageGain = new MovingAverage(period);
			var averageLoss = new MovingAverage(period);

			var previous = inputStream.First();
			foreach (var item in inputStream.Skip(1))
			{
				double gain = item.Close > previous.Close ? item.Close - previous.Close : 0.0;
				double loss = previous.Close > item.Close ? previous.Close - item.Close : 0.0;

				averageGain.Push(gain);
				averageLoss.Push(loss);

				double relativeStrength = double.IsNaN(averageGain.Current) || double.IsNaN(averageLoss.Current) ? double.NaN : averageGain.Current / averageLoss.Current;

				previous = item;
				yield return double.IsNaN(relativeStrength) ? double.NaN : 100.0 - (100.0 / (1.0 + relativeStrength));
			}
		}
	}

	public struct MacdPoint
	{
		public double Macd;
		public double Signal;
		public double Divergence;
	}

	/// <summary>
	/// Fast Moving Average class provided for example purposes on an AS IS BASIS ONLY. ACCURACY OF CALCULATION NOT GAURANTEED
	/// </summary>
	public class MovingAverage
	{
		private readonly int _length;
		private int _circIndex = -1;
		private bool _filled;
		private double _current = double.NaN;
		private readonly double _oneOverLength;
		private readonly double[] _circularBuffer;
		private double _total;

		public MovingAverage(int length)
		{
			_length = length;
			_oneOverLength = 1.0 / length;
			_circularBuffer = new double[length];
		}

		public MovingAverage Update(double value)
		{
			double lostValue = _circularBuffer[_circIndex];
			_circularBuffer[_circIndex] = value;

			// Maintain totals for Push function
			_total += value;
			_total -= lostValue;

			// If not yet filled, just return. Current value should be double.NaN
			if (!_filled)
			{
				_current = double.NaN;
				return this;
			}

			// Compute the average
			double average = 0.0;
			for (int i = 0; i < _circularBuffer.Length; i++)
			{
				average += _circularBuffer[i];
			}

			_current = average * _oneOverLength;

			return this;
		}

		public MovingAverage Push(double value)
		{
			// Apply the circular buffer
			if (++_circIndex == _length)
			{
				_circIndex = 0;
			}

			double lostValue = _circularBuffer[_circIndex];
			_circularBuffer[_circIndex] = value;

			// Compute the average
			_total += value;
			_total -= lostValue;

			// If not yet filled, just return. Current value should be double.NaN
			if (!_filled && _circIndex != _length - 1)
			{
				_current = double.NaN;
				return this;
			}
			else
			{
				// Set a flag to indicate this is the first time the buffer has been filled
				_filled = true;
			}

			_current = _total * _oneOverLength;

			return this;
		}

		public int Length { get { return _length; } }
		public double Current { get { return _current; } }
	}

}
